# Generated by Django 2.1.8 on 2019-10-06 04:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0010_checkoutstore_payment_received'),
    ]

    operations = [
        migrations.AlterField(
            model_name='checkoutstore',
            name='last_updated',
            field=models.DateTimeField(auto_now=True),
        ),
    ]
