# Generated by Django 2.1.8 on 2019-08-27 01:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0002_auto_20190813_1349'),
    ]

    operations = [
        migrations.AddField(
            model_name='productvariantrating',
            name='active_flag',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='productvariantrating',
            name='review',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='productvariantrating',
            name='review_qc_flag',
            field=models.BooleanField(default=False),
        ),
    ]
