{% extends "salebox/checkout/_base.html" %}

{% load i18n %}

{% block checkout %}
    <form method="post">
        {% csrf_token %}

        <p>Pick your shipping method:</p>
        <p>Option #1: <input type="radio" name="shipping_method" checked /></p>
        <p>Option #2: <input type="radio" name="shipping_method" /></p>
        <hr />
        <button class="btn btn-primary">{% trans "Next" %}</button>
    </form>
{% endblock %}
