{% load i18n %}

<h1>{% trans "My addresses" %}</h1>

{% if addresses|length > 0 %}
    {% include "salebox/address/list_static.html" %}
{% else %}
    <p>{% trans "No addresses available" %}</p>
{% endif %}

<hr />
<h3>{% trans "Add a new address" %}</h3>

{% if addresses|length > 0 %}
    {% include "salebox/address/add_form.html" with form=address_form form_id='addaddress' %}
{% else %}
    {% include "salebox/address/add_form.html" with form=address_form form_id='addaddress' hide_default=True %}
{% endif %}
