# Generated by Django 2.1.8 on 2019-10-05 02:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0005_productvariant_preorder_flag'),
    ]

    operations = [
        migrations.CreateModel(
            name='Translation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(max_length=7)),
                ('key', models.CharField(max_length=255)),
                ('value', models.TextField(blank=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_update', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'Translation',
                'ordering': ['language_code', 'key'],
            },
        ),
    ]
