# Generated by Django 2.1.8 on 2019-09-24 06:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0004_auto_20190829_1407'),
    ]

    operations = [
        migrations.AddField(
            model_name='productvariant',
            name='preorder_flag',
            field=models.BooleanField(default=False),
        ),
    ]
