{% spaceless %}

{% load i18n %}

<html>

<head>
    <style>
        * { margin: 0; padding: 0; }
        html, body { color: #aaa; font-size: 24px; font-family: sans-serif; height: 100%; width: 100%; }
        #msg { align-items: center; display: flex; height: 100%; justify-content: center; width: 100%; }
    </style>
</head>
<body>
    <div id="msg">{% trans "One moment please" %}</div>
    <div style="display: none;">{{ html|safe }}</div>
</body>

</html>{% endspaceless %}