{% spaceless %}

{% load i18n %}

<div class="product-basketbtn">
    {% if pv.in_wishlist %}
        <button class="product-basketbtn-empty" onclick="setWishlist({{ pv.id }}, false, this.parentNode)" onmouseover="$(this).text('{% trans "Remove" %}')" onmouseout="$(this).text('{% trans "In wishlist" %}')">{% trans "In wishlist" %}</button>
    {% else %}
        <button class="product-basketbtn-empty" onclick="setWishlist({{ pv.id }}, true, this.parentNode)">{% trans "Add to wishlist" %}</button>
    {% endif %}
    <div class="product-basketbtn-loading">
        <i class="fas fa-spinner fa-spin"></i>
    </div>
</div>

{% endspaceless %}