{% load i18n salebox_utils %}

<div class="product">
    <a href="/product/{{ pv.id }}-{{ pv.slug }}/" style="text-decoration: none">
        <div class="img">
            {% if pv.image or pv.product.image %}
                <img src="{% if pv.image %}{{ pv.image }}{% else %}{{ pv.product.image }}{% endif %}">
            {% else %}
                <div>{% trans "No picture available" %}</div>
            {% endif %}
        </div>
    </a>
    <div>
        <a href="/product/{{ pv.id }}-{{ pv.slug }}/" style="text-decoration: none">{{ p.p_name }}</a>
    </div>
    {% if pv.sale_price != pv.price %}
        <div class="price price-orig" style="text-decoration: line-through;">{{ pv.price|sb_currency }}</div>
    {% endif %}
    <div class="price">{{ pv.sale_price|sb_currency }}</div>

    {% if pv.rating_display.10 is not None %}
        {% include "salebox/utils/rating.html" with readonly=True rating=pv.rating_display.10 %}
    {% endif %}

    {% include 'salebox/basket/button_basket.html' %}
</div>
