{% extends "salebox/checkout/_base.html" %}

{% load i18n salebox_utils %}

{% block checkout %}
    <form method="post">
        {% csrf_token %}

        <p>Your Receipt</p>
        <p>Total price <strong>{{ checkout.data.basket.basket.sale_price|sb_currency }}</strong></p>
        <hr />
        <button class="btn btn-primary">{% trans "Next" %}</button>
    </form>
{% endblock %}
