# Generated by Django 2.1.8 on 2019-10-06 04:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0009_auto_20191005_1501'),
    ]

    operations = [
        migrations.AddField(
            model_name='checkoutstore',
            name='payment_received',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
