{% load i18n %}

<h1>{% trans "My account" %}</h1>
<p>{{ request.user.email }}</p>

<h3>{% trans "Shopping" %}</h3>
<ul>
    <li><a href="/basket/">{% trans "My basket" %} ({{ basket.basket.qty }})</a></li>
    <li><a href="/wishlist/">{% trans "My wishlist" %} ({{ basket.wishlist.qty }})</a></li>
    <li><a href="/my-account/address/">{% trans "My addresses" %} ({{ delivery_address_count }})</a></li>
</ul>
