{% load i18n humanize %}

<hr>

<div style="overflow: hidden">
    {% if products.count.total > 0 %}
        {% if products.count.total > 1 %}
        <span>{% blocktrans with from=products.count.from to=products.count.to total=products.count.total %}Showing {{ from }} to {{ to }} of {{ total }}{% endblocktrans %}</span>
        {% else %}
        <span>{% blocktrans with from=products.count.from total=products.count.total %}Showing {{ from }} to {{ total }}{% endblocktrans %}</span>
        {% endif %}
    {% endif %}

    <select style="float: right" onchange="setOrder(this.value)">
        {% for o in order.options %}
            <option value="{{ o.code }}"{% if o.code == order.current %} selected{% endif %}>{{ o.label }}</option>
        {% endfor %}
    </select>
</div>

<hr>

<div class="product_list">
    {% if products.count.total > 0 %}
        {% for pv in products.products %}
            {% include 'salebox/product/
            product_list_single.html' %}
        {% endfor %}
    {% else %}
        <p>{% trans "No products found" %}</p>
    {% endif %}
</div>

<hr>
