# Generated by Django 2.1.8 on 2019-10-05 05:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0006_translation'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='country',
            options={'ordering': ['code'], 'verbose_name_plural': 'Countries'},
        ),
        migrations.AlterModelOptions(
            name='countrystate',
            options={'ordering': ['full_code'], 'verbose_name_plural': 'Country States'},
        ),
        migrations.RenameField(
            model_name='country',
            old_name='code_2',
            new_name='code',
        ),
        migrations.RenameField(
            model_name='countrystate',
            old_name='code_2',
            new_name='code',
        ),
        migrations.RemoveField(
            model_name='country',
            name='code_3',
        ),
        migrations.RemoveField(
            model_name='country',
            name='name',
        ),
        migrations.RemoveField(
            model_name='countrystate',
            name='name',
        ),
        migrations.AddField(
            model_name='countrystate',
            name='full_code',
            field=models.CharField(blank=True, max_length=8, null=True),
        ),
    ]
