# Generated by Django 2.1.8 on 2019-10-05 05:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saleboxdjango', '0007_auto_20191005_1228'),
    ]

    operations = [
        migrations.AlterField(
            model_name='countrystate',
            name='code',
            field=models.CharField(blank=True, max_length=5, null=True),
        ),
    ]
