{% if readonly %}
    <div class="rating rating-{{ rating }}"></div>
{% else %}
    <div class="rating rating-vote rating-{{ rating }}" data-rating="{{ rating }}" data-id="{{ id }}"{% if onchange %} onmouseup="{{ onchange }}"{% endif %}></div>
{% endif %}
